<?php

// ===================================================================
// Global configuration parameters:

//  configure script as background process job
//  leave cpu time
//  value :=  # seconds  to sleep every # of processed lines
$backgroundmode = 1;

// define default zone
$zone = "2";

// Nodelist Data Directory if not actual script directory
// 
$ndldir = "e:/pointlst/data";


// ===================================================================


/************************************************************************/
/* BINKD_NODELISTER.PHP                                                 */
/* ====================                                                 */
/* Program to parse the raw FidoNet nodelist and spit out a             */
/* BinkD-compatible list                                                */
/*                                                                      */
/* BINKD_NODELISTER.PHP is the PHP port of the previous                 */
/* BINKD_NODELISTER.PL script v1.0                                      */
/* Copyright (c) 2002 by Jerry Schwartz and Write by Night              */
/*                                                                      */
/* BINKD_NODELISTER.PHP                                                 */
/* Copyright (c) 2007 by Ulrich Schroeter                               */
/* http://ambrosia60.dnsalias.net                                       */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/************************************************************************/

/*

#!I:/Apache/Perl/bin/perl
#
#   Program to parse the raw FidoNet nodelist and spit out a BinkD-compatible
#   list of nodes. Addresses are discovered according to these rules:
#
#   1.  the address(es) attached to the IBN flag(s),
#       which might have ports attached
#   2.  the system name field, if it has a period in the middle of it
#   3.  the address(es) attached to the INA flag(s)
#   4.  the address(es) attached to the IP flag(s)
#   5.  the "phone number", if it looks like 000 followed by an IP address
#
#==>If no address is found, an "*" is used which causes BinkD to generate
#   a host name based upon the FTN address.
#
#   @(#) binkd_nodelister.pl version 1.0 - 26 November 2002
#   @(#) Copyright (c) 2002 by Jerry Schwartz and Write by Night
 */

$versionno = "1.0";
$releasedate = "4.4.2007";
$version1 = "binkd_nodelister.php version ".$versionno." - ".$releasedate.",";
$version2 = "the php port of the well known perl binkd_nodelister.pl";
$copyright_notice1 = "binkd_nodelister.pl Copyright (c) 2002 by Jerry Schwartz and Write by Night";
$copyright_notice2 = "binkd_nodelister.php port Copyright (C) 2007 by Ulrich Schroeter";

// don't know if this conflicts with the previous licence model of Jerry Schwartz and Write by Night
$copyright_notice3 = "This is free software, and you may redistribute it under the GPL.";


/*
#
# Change History
#

#   4 April 2007 - v1.0 (PHP)
#     Add Securing BINKD.TXT output file
#        If scripts terminates execution abnormaly, the previous binkd.txt will be restored
#
#   3 April 2007 - v1.0 (PHP)
#
#     Ported script now runs
#     Eliminate 0.0.0.0 (Null) IPs
#     Eliminate dupe addresses and IPs that appears in the official
#         binkd.txt in fileecho i-binkd hatched by 1:142/928
#     Added: find actual nodelist in working dir using admin line info instead of filedate
#     Added: DNSnames validation inlc. IP address notation
#     Added: Default Zone parameter fallback if nodelist has a missing zone line
#        (i.e. Zone 2: Nodelist.075 (2007) problem)
#        comment will be added in the Error file and the Output file
#
#   22 Feb 2007 - v0.1-alpha (PHP)
#
#     Port to PHP started

#   26 Nov 2002
#
#     Version changed to 1.0, no other changes
#
#   15 Nov 2002
#
#     If no address is specified, but one or more ports are specified,
#     combine the "*" with each port
#
#     If there were multiple IBN flags, and some specified a port number and
#     others did not, redundant address*port combinations were generated
#
#     A couple of minor revisions of the comments
#
#   13 Nov 2002
#
#     Renamed program to binkd_nodelister rather than the generic parse_nodelist
#
#     Found and fixed a bug in pulling the IP address out of the phone number
#
#     Made cosmetic changes to comments so they more exactly describe the
#     program's logic
#
#   12 Nov 2002
#
#     Added support for the experimental (as/of 12 Nov 2002) INA flag
#
#     Revised the order of preference for IP addresses
#
#   1.  the address(es) attached to the IBN flag(s),
#       which might have ports attached
#   2.  the system name field, if it has a period in the middle of it
#   3.  the address(es) attached to the INA flag(s)
#   4.  the address(es) attached to the IP flag(s)
#   5.  the "phone number", if it looks like 000 followed by an IP address
#
#   02 Nov 2002
#
#     Added a check to make sure the IBN flag has additional information before
#     trying to parse it. This did no real harm, but it generated pages of
#     error messages if you ran with the -w option.
#
#     Found a typo that kept the program from recognizing an address attached
#     to an IP flag
#
#   01 Nov 2002
#
#     Now announces the version and copyright notice in the output file
#
#   31 Oct 2002
#     Major reworking of the whole scheme:
#
#     - The order of precedence for finding addresses was changed to
#
#       1.  the system name field, if it has a period in the middle of it
#       2.  the addresses attached to the IBN flag(s),
#           which might have ports attached
#       3.  the addresses attached to the IP flag(s)
#       4.  the "phone number", if it looks like 000 followed by an IP address
#
#     - If more than one port number was specified by an IBN flag, then every
#       address is combined with every port to create a * p combinations
#
#     - All of the resulting combinations are then combined into a list of
#       addresses for BinkD to try
#
#     - Internally, the possible address candidates are stored in a hash so
#       that the entire algorithm can be extended or changed more easily
#
#     - &parsewords doesn't handle single-quotes used as apostrophes in
#       nodelist entries, so it was replaced by a call to the split function.
#       This had the interesting side effect of vastly speeding up the program.
#
#   30 Oct 2002
#     If no IP address is found for a node, an * will be used but the listing
#     will be written to the error file as well
 */

$debut = time();

// set auto_detect_line_endings = ON in php.ini !!!

set_time_limit(60*60); // 1 hour ... should be enough  ;-)
$date = date("YmdHis",time());
$progress = 1;
$no_connect = 0;
$perrn = 0;
$perrt = "";
$crlf = chr(13).chr(10);

// regexp validation string for dnsnames incl. ip addresses
// $rstrdnsname = "/^[a-z0-9\-]+\.([a-z0-9\-]+\.){0,}?[a-z]{2,6}$/i";
$rstrdnsname = "/((^[a-z0-9\-]+\.([a-z0-9\-]+\.){0,}?[a-z]{2,6}$)|(^(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])$))/i";


// define("undef",false);
$undef = false;

if (!isset($REMOTE_ADDR) && !isset($_SERVER['REMOTE_ADDR'])) {
  // register_argc_argv should be set to on: if you cannot turn this option on,
  // uncommenting the next two lines. if you then get an undefine index message
  // recomment these same two lines. Also see http://www.php.net/reserved.varia
  $argc = $_SERVER["argc"];
  $argv = $_SERVER["argv"];
  $br = "\n";
  $hr = "\n-----------------------------\n";
  $run_mode = 'cgi';
  $s_yes = "+";
  $s_no  = "X";
  $s_link = "@url";
}

// echo "argv-0: ".$argv[0]."\r\n";
// echo "argv-1: ".$argv[1]."\r\n";
// echo "argv-2: ".$argv[2]."\r\n";

if (($argv[1] == "/?") || ($argv[1] == "-?") || ($argv[1] == "-h") || ($argv[1] == "--help"))
{
  print "* ".$version1."\n* ".$version2."\n* ".$copyright_notice1."\n* ".$copyright_notice2."\n\n";
  print "Usage:\n";
  print "   binkd_nodelister.php [<input nodelist> " .
         "[<output nodelist> [<error list>]]]\n\n";
  print "Parameters:\n";
  print "   <input nodelist>  a defined input nodelist i.e. nodelist.999\n";
  print "                     default: latest nodelist.### in directory\n";
  print "   <output nodelist> the output filename. default: binkd.txt\n";
  print "   <error list>      the error log filename. default: binkd.err\n";  
  print "   /? or -? or -h or --help   this helpscreen\n\n";
  die;
}


//  @unlink($target);
//  $fp=fopen($target,"w+");
//  while ( list( $key, $val ) = each( $contents ) ) {
//     fputs($fp,$val."\r\n");
//  }
//  fclose($fp);

// =============================================================================
//  subfunctions begin section

//  perl port of perl function chomp
function chomp($p) {
 $p = str_replace(chr(10),"",$p); 
 $p = str_replace(chr(13),"",$p); 
 return $p;
}


//  find position in a string
//  int position = at ( char searchstring, char string ) 
//  returns 0 if not found
function at($such,$string) {
  $slen  = strlen($such);
  $rtlen = strlen($string);
  $rtpos = 0;
  for ($j=0; $j < $rtlen; $j++) {
     $rtchr = substr($string,$j,$slen);
     if ($rtchr==$such) {
        if ($rtpos==0) {
           $rtpos = $j;
           $j = $rtlen;
        }
     }
  }
return $rtpos;
}

function ndlsearch() {
   // $sf
   global $ndldir;
   // nodelist.* => regexp: /^nodelist\.\d$/
   $rstr = "/^nodelist\.\d{3}$/i";
   $br = "\r\n";
   $onefound = false;
   $ndldays = array(
     "Monday",
     "Tuesday",
     "Wednesday",
     "Thursday",
     "Friday",
     "Saturday",
     "Sunday"
   );

   // $blockslist = array();
   $actfile = "";
   $cmpfile = chr(254);
   if ($ndldir!="") {
      $mxpath = $ndldir;
   } else {
      $mxpath = getcwd();
   }
   // echo $mxpath.$br;
   
   $actpath = $mxpath."\\";
   $blocksdir = dir($mxpath);
   while($func=$blocksdir->read()) {
     $file = strtoupper($func);
     if (($func!=".") and ($func!="..") and is_dir($actpath.$func)) {
        // is directory entry, skip
     } else {
        if (($func!=".") and ($func!="..")) {
           if (preg_match($rstr,$file)) {
             // file_get_contents -- Reads entire file into a string
             $input = file($actpath.$func);
             //print "Path: ".$actpath." File: ".$func.$br;
             // print "File: ".$func." Filesize: ".$fbbssize.$br;
             // print "1st line:".$br.$input[0].$br;
             if (substr($input[0],0,2)==";A") {
                $onefound = true;
                // is ndl file
                // print "Path: ".$actpath." File: ".$func.$br;
                // extract ndl date, NOT filedate!!!
                // get date out of admin line
                // extract string between  "for workday," and " -- Day number"
                $stop = false;
                $k = 0;
                while ((!$stop) && ($k<7)) {
                   if (at(" for ".$ndldays[$k].",",$input[0])>0) {
                      $stop = true;
                   } else {
                      $k++;
                   }
                }
                $startpos1 = at(" for ".$ndldays[$k].",",$input[0]);
                $endpos    = at(" -- Day number",$input[0]);
                $sdaylen   = strlen($ndldays[$k]);
                $linepart1 = substr($input[0],$startpos1+7+$sdaylen,$endpos-$startpos1-7-$sdaylen);
                //echo $startpos1.", ".$endpos.$br;
                //echo "[".$linepart1."]".$br;

                $uts = strtotime($linepart1);
                $ndldate = date('Ymd',$uts);
                //echo "Unix timestamp: ".$uts.$br;
                //echo "converted ....: ".$ndldate.$br;

              $actfile = $ndldate.$func;
                if ($cmpfile==chr(254)) { $cmpfile = $actfile; }
                if ($actfile > $cmpfile) {
                   $cmpfile = $actfile;
                }
             //} else {
               // skip, not a nodelist file
             }
           }
        }
     }
   }
	 closedir($blocksdir->handle);
   // name of file is crypted:  ndldate + nodelist-filename
   //  20070330nodelist.999
   // echo "Actual nodelist: ".substr($cmpfile,8).$br;

   if ($onefound) {
      return substr($cmpfile,8);
   } else {
      return false;
   }
}


// securing BINKD.TXT:  copy back bak to binkd.txt
function secbinkd($f1,$f2,$f3,$f4) {
   // $f1 = $outputfile
   // $f2 = $errorfile
   // $f3 = $ofname
   // $f4 = $targsik
   fclose($f1);
   fclose($f2);
   // securing BINKD.TXT:  copy back bak to binkd.txt
   if (file_exists($f3)) {
      unlink ($f3);
   }
   if (file_exists($f4)) {
      if (!rename($f4,$f3)) {
         // copy($ofname, $targsik)) {
         // if (!copy($targsik,$ofname)) {
         print ("failed to restore $f3...\n");
      }
   }
   return true;
}

//  subfunctions end section
// =============================================================================



//$inputfile = $argv[0];
if ( $argv[1] == "")
{
   // $default_search = "nodelist.*";
   // $source = ndlsearch();
   // securing BINKD.TXT:  nothing to do
   if ( false === ($source = ndlsearch())) {
       die ("No sourcefile found.");
   }
} else {
   $source = str_replace(chr(92),chr(47),$argv[1]);   //   \ -> / translation
}

if ($ndldir!="") {
    //   $mxpath = $ndldir;
    // expand source with ndldir
    $source = $ndldir."/".$source;
    // echo $source."\r\n";
}

if (file_exists($source)) {
    // print "The file $filename exists";
    // $inputfile = fopen($source,"r");
    $inputfile = file($source);

} else {
   // securing BINKD.TXT:  nothing to do
    die ("Can't open sourcefile: ".$source);
}

//$outputfile = (defined $argv[1]) ? ">" . $argv[1] : ">&STDOUT";
//$errorfile = (defined $argv[2]) ? ">" . $argv[2] : ">&STDERR";


$target = (($argv[2] != "") ? $argv[2] : "binkd.txt");
$ofile  = $target;
$target = str_replace(chr(92),chr(47),$target);
$ofname = $target;
$targsik = substr($target,0,at(".",$target)).".bak";


// securing BINKD.TXT:  copy to bak
if (file_exists($targsik)) {
   unlink ($targsik);
}

if (file_exists($ofname)) {
  if (!rename($ofname, $targsik)) {
     // copy($ofname, $targsik)) {
     print ("failed to copy $ofname...\n");
  }
}

$outputfile = fopen($target,"w+");

$target = (($argv[3]!="") ? $argv[3] : "binkd.err");
$efname = $target;
$target = str_replace(chr(92),chr(47),$target);
$errorfile  = fopen($target,"w+");

// echo "Outfile .: ".$ofname."\r\n";
// echo "Errorfile: ".$efname."\r\n";

/*
#   Open the input and output files
#
#   The "three-argument" format doesn't work when re-opening STDOUT as/of
#   v5.6.1 built for MSWin32-x86-multi-thread
 */

//open (INSTUFF, "<", $inputfile) || die ("Can't open $inputfile: $!");




//open (OUTSTUFF, $outputfile)    || die ("Can't open $outputfile: $!");
//open (ERRFILE, $errorfile)      || die ("Can't open $errorfile: $!");


//print (OUTSTUFF "#\tBinkD-compatible nodelist extraction - " . localtime() .
//      "\n#\n#\tCreated by $version\n#\t$copyright_notice\n#\n")
//        || die ("Can't write to $outputfile: $!");

print "* ".$version1."\n* ".$version2."\n* ".$copyright_notice1."\n* ".$copyright_notice2."\n\n";


$val = "#\tBinkD-compatible nodelist extraction - ".date("D M j G:i:s Y").$crlf."#".$crlf."#\tCreated by ".$version1.$crlf."#\t".$version2.$crlf."#\t".$copyright_notice1.$crlf."#\t".$copyright_notice2.$crlf."#".$crlf;
if ( false === fputs($outputfile,$val)) {
  // securing BINKD.TXT:  copy back bak to binkd.txt
  secbinkd($outputfile,$errorfile,$ofname,$targsik);
  die ("Can't write to outfile: ".$ofname);
}


//#   Loop through the raw nodelist, passing through comment (;A and ;S) lines


$num = count($inputfile);
$ocount = 0;
$ncomment = 0;

// $num = 165;   // DEBUG: first 165 lines only

echo "# lines in source (".$source."): \t".$num."\r\n";

// define default Zone aka  i.e. 2:2/0  as fallback
$net = $zone;
$node = "0";

$n10th = intval($num / 10);
$bgmc = 1;

$fbtrig = -1;

for($i=0;$i<$num;$i++) {

  $inline = $inputfile[$i];

  if (($backgroundmode > 0) AND ($i!=0)) {
    if ($i == ($n10th*$bgmc)) {
       // sleep for i.e. 2 seconds every # of lines
       sleep($backgroundmode);
       $bgmc++;
    }
  }
  
  // while (!feof ($inputfile)) {
  //  $buffer = fgets($inputfile, 4096);

  // while ($inline = <INSTUFF>)
  //{

  $inline = chomp($inline);

  if (!preg_match("/^\;/", $inline)) {   // #   For non-comment lines, parse out all of the fields in the listing
    $listing = explode(",",$inline);

    // GET_ADDRESS:
    {   // #   Pick off the FTN address
      if (strtolower($listing[0]) == "zone") { //   && do {   #   ZCs are a special case
                                      $zone = $listing[1];
                                      $net = $zone;
                                      $node = "0";
                                      // last GET_ADDRESS;
                                      $GET_ADDRESS = $zone.":".$net."/".$node;
                                      $fbtrig = ($fbtrig==-1) ? 0 : 2;
      } elseif (strtolower($listing[0]) == "region") { // && do {   #   RCs are also a special case
                                      $net = $listing[1];
                                      $node = "0";
                                      // last GET_ADDRESS;
                                      $GET_ADDRESS = $zone.":".$net."/".$node;
                                      $fbtrig = ($fbtrig==-1) ? 1 : 2;
      } elseif (strtolower($listing[0]) == "host") { //   && do {   #   and NCs are special, too
                                      $net = $listing[1];
                                      $node = "0";
                                      //last GET_ADDRESS;
                                      $GET_ADDRESS = $zone.":".$net."/".$node;
                                      $fbtrig = ($fbtrig==-1) ? 1 : 2;
      } else {
         $node = $listing[1];  //   #   Otherwise, the second field is the node number
         $GET_ADDRESS = $zone.":".$net."/".$node;
         $fbtrig = ($fbtrig==-1) ? 1 : 2;
      }
    }

    if ($fbtrig==1) {                                       //  $fbtrig = (-1) ? 1 : 2;
      if (false === fputs($errorfile,"Zoneline missing, fallback Zone\t$zone\r\n")) {
          // securing BINKD.TXT:  copy back bak to binkd.txt
          secbinkd($outputfile,$errorfile,$ofname,$targsik);
          die("Can't write to errorfile: ".$efname."!");
      }            
      if (false === fputs($outputfile,"# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n# Zoneline missing, fallback Zone\t$zone\r\n# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n")) {
          // securing BINKD.TXT:  copy back bak to binkd.txt
          secbinkd($outputfile,$errorfile,$ofname,$targsik);
          die("Can't write to outputfile: ".$ofname."!");
      }             
    }


#   Let's find the candidates for an IP address or host name

    // $possible_addresses = array();
    $possible_addresses =   array(
          "system_name"   =>  "",
          "ibn_flag"      =>  "",
          "ina_flag"      =>  "",
          "ip_flag"       =>  "",
          "phone_number"  =>  ""
                            );
                                                        
    $all_addresses = array();                        
                            
    $port_list = array();
    $address = "";


// #   Now, look for an IBN flag signalling that this is a BinkP node

    $ibn_flag_count = count(preg_grep ("/^IBN/i", $listing));     //  #   Any IBN flags at all?
    if ($ibn_flag_count) {
      $ibn_flag = preg_grep ("/^IBN/i", $listing);   //  #   Get the flag(s)
      foreach ($ibn_flag as $this_ibn_flag) {
        // #   The address might be in the IBN flag itself
        if (isset($possible_address_port)) {unset($possible_address_port); }
        if (preg_match("/^IBN:(.+)/i",$this_ibn_flag,$matches)) {
           $possible_address_port = $matches[1];
        } else {
           // no address, no port in IBN flag
           $possible_address_port = "";
        }

// #   This might be an address, a port number, an address:port combo,
// #   or undefined; let's find out. If there is more than one IBN flag with an
// #   address, we string them together with semicolons as BinkD specifies.
// #
// #   We also have to build a list of the specified ports, for later use.
// #
// #=> There is an important ambiguity here. If there is a single IBN flag with a
// #   port number, then we can assume it applies to every address; if there
// #   is more than one IBN flag with both a port number and an address,
// #   we can apply each port number to the associated address. The ambiguity
// #   arises if there is more than one IBN flag with a port, but not every IBN
// #   flag has both a port and an address. We also have to decide what to do with
// #   any other addresses we might have found.
// #
// #   This program will assume that each port applies to each address (except when
// #   explicitly paired), so it will produce multiple address entries: one for
// #   each combination of port and address.
// #
// #   Thank goodness the only valid place to specify a port is in the IBN flag
// #   (I hope).


        if ($possible_address_port!="") {
           // irgendetwas gefunden
           if ( preg_match("/(.+):(\d+)$/", $possible_address_port, $matches) ) {
              //  address & port

              $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$matches[1])."$/i", $all_addresses));  //  #   Any dupe?
              if ($dupe_count==0) {
                if (preg_match($rstrdnsname, $matches[1])) {  // is valid dnsname?
                   if ($possible_addresses["ibn_flag"]!="") {
                      $possible_addresses["ibn_flag"] = $possible_addresses["ibn_flag"] . ";" . $matches[1]. ":" . $matches[2];
                      $all_addresses[count($all_addresses) + 1] = $matches[1];
                   } else {
                      $possible_addresses["ibn_flag"] = $matches[1] . ":" . $matches[2];
                      $all_addresses[count($all_addresses) + 1] = $matches[1];
                   }
                }
              }
              
              $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$matches[2])."$/i", $port_list));  //  #   Any dupe?
              if ($dupe_count==0) {
                $port_list[count($port_list) + 1] = $matches[2];
              }
              //last GET_PORT;
              //$GET_PORT = $matches[2];
           } else {
              // only address or only port
              if ( preg_match("/^(\d+)$/", $possible_address_port, $matches) ) {
                // only port
                $dupe_count = count(preg_grep ("/^".$matches[1]."$/i", $port_list));  //  #   Any dupe?
                if ($dupe_count==0) {
                  $port_list[count($port_list) + 1] = $matches[1];
                }
                // last GET_PORT;
                //$GET_PORT = $matches[1];
              } else {
                // only address
                if ( preg_match("/\D/", $possible_address_port) ) {
                   $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$possible_address_port)."$/i", $all_addresses));  //  #   Any dupe?
                   if ($dupe_count==0) {
                      if (preg_match($rstrdnsname, $matches[1])) {  // is valid dnsname? 
                         if ($possible_addresses["ibn_flag"]!="") {
                            $possible_addresses["ibn_flag"] = $possible_addresses["ibn_flag"] . ";" . $possible_address_port;
                            $all_addresses[count($all_addresses) + 1] = $possible_address_port;
                            // last GET_PORT;
                            //$GET_PORT = $possible_address_port;
                         } else {
                            $possible_addresses["ibn_flag"] = $possible_address_port;
                            $all_addresses[count($all_addresses) + 1] = $possible_address_port;
                            // last GET_PORT;
                            //$GET_PORT = $possible_address_port;
                         }
                      }
                   } 
                }
              }
           }
        }
      }  // foreach ibn_flag  	
        

// #   The system name might be the address, if it has a . in the middle of it

      if (preg_match("/.+\..+/", $listing[2])) {
          $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$listing[2])."$/i", $all_addresses));  //  #   Any dupe?
          if ($dupe_count==0) {
             if (preg_match($rstrdnsname, $listing[2])) {  // is valid dnsname?
                $possible_addresses["system_name"] =  $listing[2];
                $all_addresses[count($all_addresses) + 1] = $listing[2];
             }
          }
      } else {
          $possible_addresses["system_name"] =  "";
      }

// #   The address might also be in the phone number; if it begins with 000 and
// #   has four more numeric parts, assume that it is an IP address

      if (preg_match("/^000[-|\.](\d+)[-|\.](\d+)[-|\.](\d+)[-|\.](\d+)/x", $listing[5], $matches)) {
          $tmp_string = $matches[1] . "." . $matches[2] . "." . $matches[3] . "." . $matches[4];
          $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$tmp_string)."$/i", $all_addresses));  //  #   Any dupe?
          if ($dupe_count==0) {
             $possible_addresses["phone_number"] =  $tmp_string;
             $all_addresses[count($all_addresses) + 1] = $tmp_string;
          }
      } else {
          $possible_addresses["phone_number"] = "";       
      }

// #   The address might be in an INA flag

      $ina_flag_count = count(preg_grep ("/^INA/i", $listing));  //  #   Any INA flags?
      if ($ina_flag_count) {
         $ina_flag = preg_grep ("/^INA/i", $listing);  //  #   Get all of the flags
         //  INA:address
         //  INA:address:port
         if (isset($possible_address_port)) {unset($possible_address_port); }
         foreach  ($ina_flag as $this_ina_flag) { // #   Pick off and concatenate the addresses (and previously-found
           if (preg_match("/^INA:(.+)/i",$this_ina_flag,$matches)) {
              $possible_address_port = $matches[1];
           } else {
              // no address, no port in INA flag
              $possible_address_port = "";
           }

           if ($possible_address_port!="") {
              // irgendetwas gefunden
              if ( preg_match("/(.+):(\d+)$/", $possible_address_port, $matches) ) {
                 //  address & port

                 $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$matches[1])."$/i", $all_addresses));  //  #   Any dupe?
                 if ($dupe_count==0) {
                    if (preg_match($rstrdnsname, $matches[1])) {  // is valid dnsname?                  
                       if ($possible_addresses["ina_flag"]!="") {
                          $possible_addresses["ina_flag"] = $possible_addresses["ina_flag"] . ";" . $matches[1]. ":" . $matches[2];
                          $all_addresses[count($all_addresses) + 1] = $matches[1];
                       } else {
                          $possible_addresses["ina_flag"] = $matches[1] . ":" . $matches[2];
                          $all_addresses[count($all_addresses) + 1] = $matches[1];
                       }
                    }
                 }
              
                 $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$matches[2])."$/i", $port_list));  //  #   Any dupe?
                 if ($dupe_count==0) {
                    $port_list[count($port_list) + 1] = $matches[2];
                 }
              } else {
                 // only address or only port
                 if ( preg_match("/^(\d+)$/", $possible_address_port, $matches) ) {
                    // only port
                    $dupe_count = count(preg_grep ("/^".$matches[1]."$/i", $port_list));  //  #   Any dupe?
                    if ($dupe_count==0) {
                       $port_list[count($port_list) + 1] = $matches[1];
                    }
                 } else {
                    // only address
                    if ( preg_match("/\D/", $possible_address_port) ) {
                       $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$possible_address_port)."$/i", $all_addresses));  //  #   Any dupe?
                       if ($dupe_count==0) {
                          if (preg_match($rstrdnsname, $possible_address_port)) {  // is valid dnsname?                  
                             if ($possible_addresses["ina_flag"]!="") {
                                $possible_addresses["ina_flag"] = $possible_addresses["ina_flag"] . ";" . $possible_address_port;
                                $all_addresses[count($all_addresses) + 1] = $possible_address_port;
                             } else {
                                $possible_addresses["ina_flag"] = $possible_address_port;
                                $all_addresses[count($all_addresses) + 1] = $possible_address_port;
                             }
                          }
                       }
                    }
                  }
              }
           }
         }
       }
         
// #   The address might be in an IP flag

       $ip_flag_count = count(preg_grep ("/^IP/i", $listing));  //  #   Any IP flags?
       if ($ip_flag_count) {
         $ip_flag = preg_grep ("/^IP/i", $listing);   //  #   Get all of the flags
         //  IP:address
         //  IP:address:port
         if (isset($possible_address_port)) {unset($possible_address_port); }
         foreach ($ip_flag as $this_ip_flag) { // #   Pick off and concatenate the addresses (and previously-found
           if (preg_match("/^IP:(.+)/i",$this_ip_flag,$matches)) {
              $possible_address_port = $matches[1];
           } else {
              // no address, no port in IP flag
              $possible_address_port = "";
           }

           if ($possible_address_port!="") {
              // irgendetwas gefunden
              if ( preg_match("/(.+):(\d+)$/", $possible_address_port, $matches) ) {
                 //  address & port

                 $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$matches[1])."$/i", $all_addresses));  //  #   Any dupe?
                 if ($dupe_count==0) {
                    if (preg_match($rstrdnsname, $matches[1])) {  // is valid dnsname?                  
                       if ($possible_addresses["ip_flag"]!="") {
                          $possible_addresses["ip_flag"] = $possible_addresses["ip_flag"] . ";" . $matches[1]. ":" . $matches[2];
                          $all_addresses[count($all_addresses) + 1] = $matches[1];
                       } else {
                          $possible_addresses["ip_flag"] = $matches[1] . ":" . $matches[2];
                          $all_addresses[count($all_addresses) + 1] = $matches[1];
                       }
                    }
                 }
              
                 $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$matches[2])."$/i", $port_list));  //  #   Any dupe?
                 if ($dupe_count==0) {
                    $port_list[count($port_list) + 1] = $matches[2];
                 }
              } else {
                 // only address or only port
                 if ( preg_match("/^(\d+)$/", $possible_address_port, $matches) ) {
                    // only port
                    $dupe_count = count(preg_grep ("/^".$matches[1]."$/i", $port_list));  //  #   Any dupe?
                    if ($dupe_count==0) {
                       $port_list[count($port_list) + 1] = $matches[1];
                    }
                 } else {
                    // only address
                    if ( preg_match("/\D/", $possible_address_port) ) {
                       $dupe_count = count(preg_grep ("/^".str_replace(".","\.",$possible_address_port)."$/i", $all_addresses));  //  #   Any dupe?
                       if ($dupe_count==0) {
                          if (preg_match($rstrdnsname, $possible_address_port)) {  // is valid dnsname?                       
                             if ($possible_addresses["ip_flag"]!="") {
                                $possible_addresses["ip_flag"] = $possible_addresses["ip_flag"] . ";" . $possible_address_port;
                                $all_addresses[count($all_addresses) + 1] = $possible_address_port;
                             } else {
                                $possible_addresses["ip_flag"] = $possible_address_port;
                                $all_addresses[count($all_addresses) + 1] = $possible_address_port;
                             }
                          }
                       }
                    }
                  }
              }
           }
         }
       }
      

// #   By now, we should have an Internet address from at least one of four
// #   places; now we concatenate the possibilities in this order:
// #
// #   1.  the address(es) attached to the IBN flag(s),
// #       which might have ports attached
// #   2.  the system name field, if it has a period in the middle of it
// #   3.  the address(es) attached to the INA flag(s)
// #   4.  the address(es) attached to the IP flag(s)
// #   5.  the "phone number", if it looks like 000 followed by an IP address
// #
// #   Remember, we still have (maybe) a list of ports

// #   Do the IBN flags, which would already have ports attached if found

      if ($possible_addresses["ibn_flag"]!="") {   // #   Got any?
         if ($address!="") {
            $address = $address . ";" . $possible_addresses["ibn_flag"];
         } else {
            $address = $possible_addresses["ibn_flag"];
         }
      }
      
// #   Do the system name * however many ports

      if ($possible_addresses["system_name"]!="") {   // # Looked like an address?
         if ((count($port_list) >=0) AND ($port_list[1]!="")) {   // #   Any ports?
           foreach ($port_list as $this_port) {
             if ($address!="") {
                $address = $address . ";" . $possible_addresses["system_name"] . ":" . $this_port;
             } else {
                $address = $possible_addresses["system_name"] . ":" . $this_port;
             }
           }
         } else {   //  #    No ports, just use the name
           if ($address!="") {
              $address = $address . ";" . $possible_addresses["system_name"];
           } else {
              $address = $possible_addresses["system_name"];
           }
         }
      }

// #   Now do the INA flag(s), which would already have ports attached if found

      if ($possible_addresses["ina_flag"]!="") {   //  #   Got any?
         if ($address!="") {
            $address = $address . ";" . $possible_addresses["ina_flag"];
         } else {
            $address = $possible_addresses["ina_flag"]; 
         }
      }


// #   Now do the IP flags, which would already have ports attached if found

      if ($possible_addresses["ip_flag"]!="") {  //  #   Got any?
         if ($address!="") {
            $address = $address . ";" . $possible_addresses["ip_flag"];
         } else {
            $address = $possible_addresses["ip_flag"];
         }
      }
      
// #   Now do the phone number * number of ports

      if (($possible_addresses["phone_number"]!="") AND ($possible_addresses["phone_number"]!="0.0.0.0")) { // # Looked like an address?
         if ((count($port_list) >=0) AND ($port_list[1]!="")) {   // #   Any ports?
           foreach ($port_list as $this_port) {
             if ($address!="") {
                $address = $address . ";" . $possible_addresses["phone_number"]. ":" . $this_port;
             } else {
                $address = $possible_addresses["phone_number"]. ":" . $this_port;
             }
           }
         } else {  // #    No ports, just use the IP address
           if ($address!="") {
              $address = $address . ";" . $possible_addresses["phone_number"];
           } else {
              $address = $possible_addresses["phone_number"];
           }
         }
      }

// #   Did we ever find any addresses at all?

      if ($address == "") {
         if (false === fputs($errorfile,"Addr? $zone:$net/$node\t$inline\r\n")) {
            // securing BINKD.TXT:  copy back bak to binkd.txt
            secbinkd($outputfile,$errorfile,$ofname,$targsik);
            die("Can't write to errorfile: ".$efname."!");
         }            

// #       No addresses at all, so use "*"; but there might be ports

         if ((count($port_list) >=0) AND ($port_list[1]!="")) { //  #   Any ports?
           // DEBUG
           // echo "count(port_list): ".count($port_list)."\r\n";
           foreach ($port_list as $this_port) {
             if ($address!="") {
               $address = $address . ";*" . ":" . $this_port;
             } else {
               $address =  "*" . ":" . $this_port;            
             }
           }
         } else {  // #    No ports, just use the "*"
           // DEBUG
           // echo "count(port_list): ".count($port_list)."\r\n";
           $address = "*";
         }
      }

     // DEBUG
     // echo "system_name : ".$possible_addresses["system_name"]."\r\n";
     // echo "ibn_flag ...: ".$possible_addresses["ibn_flag"]."\r\n";
     // echo "ina_flag ...: ".$possible_addresses["ina_flag"]."\r\n";
     // echo "ip_flag ....: ".$possible_addresses["ip_flag"]."\r\n";
     // echo "phone_number: ".$possible_addresses["phone_number"]."\r\n";

      $ocount++;
      $ncomment++;
      if (false === fputs($outputfile,"Node $zone:$net/$node\t$address\t-\r\n")) {
         $ocount--;
         $ncomment--;
         // securing BINKD.TXT:  copy back bak to binkd.txt
         secbinkd($outputfile,$errorfile,$ofname,$targsik);
         die("Can't write to outputfile: ".$ofname."!");
      }             

    }  //  if ibn_flag, no ibn flag, no listing
  } elseif (preg_match("/^;[A|S](.*)/i",$inline,$matches)) {
       $ocount++;
       if (false === fputs($outputfile,"#".$matches[1]."\r\n")) {
          $ocount--;
          // securing BINKD.TXT:  copy back bak to binkd.txt
          secbinkd($outputfile,$errorfile,$ofname,$targsik);
          die("Can't write to outputfile: ".$ofname."!");
       }            
  }
}

echo "# total lines in output (".$ofname."): \t".$ocount."\r\n";
echo "# IBN lines in output (".$ofname."): \t".$ncomment."\r\n";
echo "# comment lines in output (".$ofname."): \t".($ocount-$ncomment)."\r\n";

fclose($outputfile);
fclose($errorfile);
  
?>
